@@ -23,8 +23,6 @@ _____       _    _    Corso   Italia,  178
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 static char rcsid[] = "$Id$";
-# include <sql.h>
-# include <sqlext.h>
 # ifdef WIN32
 # include <windows.h>
 static char User[9 + 1];
@@ -35,6 +33,8 @@ extern "C" HINSTANCE ghInstance;
 # include <string.h>
 # include <ctype.h>
 # endif
+# include <sql.h>
+# include <sqlext.h>
 # include "pstring.h"
 # include "envirini.h"
 # include "dberror.h"
@@ -47,12 +47,13 @@ extern "C" {
 #include "except.h"
 }
 
-const int DEFAULT_PORT = 9876;
+const int DEFAULT_PORT = 6789;
 
 
 extern "C" {
 
 extern CorbaMethod methods[];
+# ifndef WIN32
 void i_SQLError (void ** argv) {}
 void i_SQLTables (void ** argv) {}
 void i_SQLPrepare (void ** argv) {}
@@ -102,14 +103,230 @@ void i_SQLMoreResults (void ** argv) {}
 void i_SQLForeignKeys (void ** argv) {}
 void i_SQLDescribeParam (void ** argv) {}
 void i_SQLColumnPrivileges (void ** argv) {}
+# endif
+
+static RETCODE mySQLAllocConnect (HENV henv, HDBC FAR * phdbc);
+static RETCODE mySQLAllocEnv(HENV FAR * phenv);
+static RETCODE mySQLFreeEnv (HENV henv);
+static RETCODE mySQLFreeConnect (HDBC hdbc);
+
+//  Allocate a SQL statement
+
+static RETCODE
+mySQLAllocStmt(
+    HDBC       hdbc,
+    HSTMT FAR *phstmt)
+{
+   CORBA_Long Return = SQL_ERROR;
+   ClntHstmt *myHstmt;
+   ClntHdbc *myHdbc = (ClntHdbc *) hdbc;
+   void *argv[3];
+
+   if (!myHdbc || !myHdbc->IsA(ClntHdbc::Class))
+      return SQL_INVALID_HANDLE;
+   DelError (0, myHdbc, 0);
+
+   myHstmt = new ClntHstmt(myHdbc);
+   myHstmt->hdbc = myHdbc;
+   argv[0] = &Return;
+   argv[1] = (void *) &myHdbc->srvHdbc;
+   argv[2] = &myHstmt->srvHstmt;
+
+   Exception *ex = Exception_new();
+   if (doCall (myHdbc, ex, "SQLAllocStmt", argv)) {
+      if (Return == SQL_SUCCESS) {
+         *phstmt =  (HSTMT) myHstmt;
+      } else
+         delete myHstmt;
+   }
+   Exception_delete (ex);
+   return (RETCODE) Return;
+}
+
+RETCODE SQL_API SQLAllocStmt(
+    HDBC       hdbc,
+    HSTMT FAR *phstmt)
+{
+   return mySQLAllocStmt (hdbc, phstmt);
+}
+
+static RETCODE
+mySQLFreeStmt(
+    HSTMT   hstmt,
+    UWORD   fOption)
+{
+   CORBA_Long Return = SQL_ERROR;
+   ClntHstmt *myHstmt = (ClntHstmt *) hstmt;
+   void *argv[3];
+
+   if (!myHstmt || !myHstmt->IsA(ClntHstmt::Class))
+      return SQL_INVALID_HANDLE;
+   DelError (0,0,myHstmt);
+
+   argv[0] = &Return;
+   argv[1] = &myHstmt->srvHstmt;
+   argv[2] = &fOption;
+
+   Exception *ex = Exception_new();
+   if (doCall (myHstmt, ex, "SQLFreeStmt", argv) && Return == SQL_SUCCESS) {
+      if (fOption == SQL_DROP)
+         delete myHstmt;
+      else if (fOption == SQL_UNBIND) {
+         BindCol *bc;
+         for (int i = 0; i < myHstmt->cols.size(); i++)
+            if ((bc = myHstmt->cols.getAt(i)) != 0) {
+               delete bc;
+               myHstmt->cols.setAt (i, (BindCol *) 0);
+            }
+      } else if (fOption == SQL_RESET_PARAMS) {
+         Parameter *p;
+         for (int i = 0; i < myHstmt->param.size(); i++)
+            if ((p = myHstmt->param.getAt(i)) != 0) {
+               delete p;
+               myHstmt->param.setAt (i, (Parameter *) 0);
+            }
+      }
+   }
+   Exception_delete (ex);
+   return (RETCODE) Return;
+}
+
+RETCODE SQL_API SQLFreeStmt(
+    HSTMT   hstmt,
+    UWORD   fOption)
+{
+   return mySQLFreeStmt (hstmt, fOption);
+}
+
+//   Allocates an environment, connection, statement, or descriptor handles
+
+/*
+SQLRETURN SQL_API
+SQLAllocHandle (SQLSMALLINT handleType,
+                SQLHANDLE inputHandle,
+                SQLHANDLE * outputHandlePtr)
+{
+   SQLRETURN Return;
+
+   switch (handleType) {
+   case SQL_HANDLE_ENV:
+      Return = mySQLAllocEnv (outputHandlePtr);
+      break;
+   case SQL_HANDLE_DBC:
+      Return = mySQLAllocConnect (inputHandle, outputHandlePtr);
+      break;
+   case SQL_HANDLE_STMT:
+      Return = mySQLAllocStmt (inputHandle, outputHandlePtr);
+      break;
+   case SQL_HANDLE_DESC:
+   default:
+      Return = SQL_ERROR;
+      break;
+   }
+   return Return;
+}
+
+SQLRETURN SQL_API
+SQLFreeHandle (SQLSMALLINT handleType,
+               SQLHANDLE handle)
+{
+   SQLRETURN Return;
+
+   switch (handleType) {
+   case SQL_HANDLE_ENV:
+      Return = mySQLFreeEnv (handle);
+      break;
+   case SQL_HANDLE_DBC:
+      Return = mySQLFreeConnect (handle);
+      break;
+   case SQL_HANDLE_STMT:
+      Return = mySQLFreeStmt (handle, SQL_DROP);
+      break;
+   case SQL_HANDLE_DESC:
+   default:
+      Return = SQL_ERROR;
+      break;
+   }
+   return Return;
+}
+*/
+
+// Sets attributes that govern aspects of environments
+
+SQLRETURN SQL_API
+SQLSetEnvAttr(SQLHENV environmentHandle,
+              SQLINTEGER attribute,
+              SQLPOINTER valuePtr,
+              SQLINTEGER stringLength)
+{
+   SQLRETURN Return = SQL_SUCCESS;
+   ClntHenv *env = (ClntHenv *)environmentHandle;
+
+   if (env == 0 || !env->IsA(ClntHenv::Class))
+      Return = SQL_INVALID_HANDLE;
+   else {
+      switch (attribute) {
+      case SQL_ATTR_ODBC_VERSION:
+         Return = (SQLRETURN) env->setVersion ((long) valuePtr);
+         break;
+      case SQL_ATTR_OUTPUT_NTS:
+         if ((long) valuePtr != SQL_TRUE) {
+            AddError (env, 0, 0, "HYC00", 0,
+                        "Optional feature not implemented");
+            Return = SQL_ERROR;
+         }
+         break;
+      case SQL_ATTR_CP_MATCH:
+      case SQL_ATTR_CONNECTION_POOLING:
+      default:
+         AddError (env, 0, 0, "HYC00", 0,
+                        "Optional feature not implemented");
+         Return = SQL_ERROR;
+         break;
+      }
+   }
+   return Return;
+}
 
 
+// Returns the current setting of an environment attribute
+
+SQLRETURN SQL_API
+SQLGetEnvAttr(SQLHENV environmentHandle,
+              SQLINTEGER attribute,
+              SQLPOINTER valuePtr,
+              SQLINTEGER bufferLength,
+              SQLINTEGER *stringLengthPtr)
+{
+   SQLRETURN Return = SQL_SUCCESS;
+   ClntHenv *env = (ClntHenv *)environmentHandle;
+
+   if (env == 0 || !env->IsA(ClntHenv::Class))
+      Return = SQL_INVALID_HANDLE;
+   else {
+      switch (attribute) {
+      case SQL_ATTR_ODBC_VERSION:
+         valuePtr = (void*) env->getVersion ();
+         break;
+      case SQL_ATTR_OUTPUT_NTS:
+         valuePtr = (void*) SQL_TRUE;
+         break;
+      case SQL_ATTR_CP_MATCH:
+      case SQL_ATTR_CONNECTION_POOLING:
+      default:
+         AddError (env, 0, 0, "HYC00", 0,
+                        "Optional feature not implemented");
+         Return = SQL_ERROR;
+         break;
+      }
+   }
+   return Return;
+}
 
 //   Allocate an environment (ENV) block.
 
-
-RETCODE SQL_API SQLAllocEnv(
-   HENV FAR * phenv)
+static RETCODE
+mySQLAllocEnv(HENV FAR * phenv)
 {
    ClntHenv *henv =  new ClntHenv;
    *phenv = (HENV) henv;
@@ -118,11 +335,16 @@ RETCODE SQL_API SQLAllocEnv(
 # endif
    return SQL_SUCCESS;
 }
+
+RETCODE SQL_API SQLAllocEnv (HENV FAR * phenv)
+{
+   return mySQLAllocEnv (phenv);
+}
+
 //   Allocate a DBC block.
 
-RETCODE   SQL_API SQLAllocConnect(
-   HENV   henv,
-   HDBC FAR * phdbc)
+static RETCODE
+mySQLAllocConnect (HENV henv, HDBC FAR * phdbc)
 {
    ClntHdbc *hdbc;
    ClntHenv *myHenv = (ClntHenv *) henv;
@@ -139,14 +361,15 @@ RETCODE   SQL_API SQLAllocConnect(
    return SQL_SUCCESS;
 }
 
-static RETCODE mySQLConnect(
-   HDBC   hdbc,
-   UCHAR FAR *szDSN,
-   SWORD   cbDSN,
-   UCHAR FAR *szUID,
-   SWORD   cbUID,
-   UCHAR FAR *szAuthStr,
-   SWORD   cbAuthStr)
+RETCODE SQL_API SQLAllocConnect(HENV henv, HDBC FAR * phdbc)
+{
+   return mySQLAllocConnect(henv, phdbc);
+}
+
+static RETCODE
+mySQLConnect(HDBC hdbc, UCHAR FAR *szDSN, SWORD cbDSN,
+                        UCHAR FAR *szUID, SWORD   cbUID,
+                        UCHAR FAR *szAuthStr, SWORD cbAuthStr)
 {
    ClntHdbc *myHdbc = (ClntHdbc *) hdbc;
    CORBA_Long Return = SQL_ERROR;
@@ -154,7 +377,7 @@ static RETCODE mySQLConnect(
    char *nomeDB = 0;
    unsigned char *user = 0;
    unsigned char *password = 0;
-   int sql_nts = SQL_NTS;
+   CORBA_Short sql_nts = SQL_NTS;
    EnvirIni odbcEnv;
 
    if (!myHdbc || !myHdbc->IsA(ClntHdbc::Class)) 
@@ -193,21 +416,11 @@ static RETCODE mySQLConnect(
       void *argv[8];
       argv[0] = &Return;
       argv[1] = &myHdbc->henv->srvHenv;
-      if (!Invoke_invoke (myHdbc->call, ex, "SQLAllocEnv", argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (myHdbc->henv, 0, 0, stat, ex->code, ex->description);
-         Exception_log(ex);
-      } else {
+      if (doCall (myHdbc, ex, "SQLAllocEnv", argv)) {
          argv[0] = &Return;
          argv[1] = &myHdbc->henv->srvHenv;
          argv[2] = &myHdbc->srvHdbc;
-         if (!Invoke_invoke (myHdbc->call, ex, "SQLAllocConnect", argv)) {
-            char stat[6];
-            sprintf (stat,"08%03d", ex->code);
-            AddError (0, myHdbc, 0, stat, ex->code, ex->description);
-            Exception_log(ex);
-         } else {
+         if (doCall (myHdbc, ex, "SQLAllocConnect", argv)) {
             if (Return == SQL_SUCCESS) {
                argv[1] = &myHdbc->srvHdbc;
                argv[2] = myHdbc->name.gets();
@@ -216,12 +429,7 @@ static RETCODE mySQLConnect(
                argv[5] = &cbUID;
                argv[6] = password;
                argv[7] = &cbAuthStr;
-               if (!Invoke_invoke (myHdbc->call, ex, "SQLConnect", argv)) {
-                  char stat[6];
-                  sprintf (stat,"08%03d", ex->code);
-                  AddError (0, myHdbc, 0, stat, ex->code, ex->description);
-                  Exception_log(ex);
-               }
+               doCall (myHdbc, ex, "SQLConnect", argv);
             }
          }
       }
@@ -302,6 +510,7 @@ RETCODE   SQL_API SQLDriverConnect(
    PBool   fPrompt = PFalse;
    char *puntdsn = (char *)0;
    char *semicolon = (char *)0;
+   int iRet;
    char ProfString[10]; 
    char ProfStringUPPER[10];
    PBool PasswordRequest = PFalse;
@@ -409,7 +618,10 @@ RETCODE   SQL_API SQLDriverConnect(
                sprintf ((char *) puntdsn, ";PWD=%s", nomePWD.gets());
          }
          if (pcbConnStrOut != (SWORD FAR *) 0)
-            *pcbConnStrOut = strlen((char *) szConnStrOut);
+            if (szConnStrOut)
+               *pcbConnStrOut = strlen((char *) szConnStrOut);
+            else
+               *pcbConnStrOut = 0;
       }
    }
    return (RETCODE) Return;
@@ -427,17 +639,12 @@ RETCODE SQL_API SQLDisconnect(
    argv[1] = &myHdbc->srvHdbc;
    Exception *ex = Exception_new();
 /* Ci sarebbe anche da interrompere la connessione !! */
-   if (!Invoke_invoke (myHdbc->call, ex, "SQLDisconnect", argv)) {
-      char stat[6];
-      sprintf (stat,"08%03d", ex->code);
-      AddError (0, myHdbc, 0, stat, ex->code, ex->description);
-      Exception_log(ex);
-   }
+   doCall (myHdbc, ex, "SQLDisconnect", argv);
    Exception_delete (ex);
    return (RETCODE) Return;
 }
 
-RETCODE   SQL_API SQLFreeConnect(
+static RETCODE mySQLFreeConnect(
    HDBC   hdbc)
 {
    ClntHdbc *myHdbc = (ClntHdbc *) hdbc;
@@ -447,24 +654,22 @@ RETCODE   SQL_API SQLFreeConnect(
    argv[0] = &Return;
    argv[1] = &myHdbc->srvHdbc;
    Exception *ex = Exception_new();
-   if (!Invoke_invoke (myHdbc->call, ex, "SQLFreeConnect", argv)) {
-      char stat[6];
-      sprintf (stat,"08%03d", ex->code);
-      AddError (0, myHdbc, 0, stat, ex->code, ex->description);
-      Exception_log(ex);
-   } else {
-      if (Return == SQL_SUCCESS) {
-         IiopObj_delete (myHdbc->obj);
-         Invoke_delete (myHdbc->call);
-         delete myHdbc;
-      }
+   if (doCall (myHdbc, ex, "SQLFreeConnect", argv) && Return == SQL_SUCCESS) {
+      IiopObj_delete (myHdbc->obj);
+      Invoke_delete (myHdbc->call);
+      delete myHdbc;
    }
    Exception_delete (ex);
    return (RETCODE) Return;
 }
+
+RETCODE SQL_API SQLFreeConnect (HDBC hdbc)
+{
+   return mySQLFreeConnect(hdbc);
+}
  
 
-RETCODE SQL_API SQLFreeEnv(
+static RETCODE mySQLFreeEnv(
    HENV   henv)
 {
    ClntHenv *myHenv = (ClntHenv *) henv;
@@ -474,6 +679,11 @@ RETCODE SQL_API SQLFreeEnv(
    return SQL_SUCCESS;
 }
 
+RETCODE SQL_API SQLFreeEnv (HENV henv)
+{
+   return mySQLFreeEnv (henv);
+}
+
 /*
 RETCODE   SQL_API SQLBrowseConnect(
    HDBC   hdbc,
