@@ -4,30 +4,34 @@ _____       _    _    Corso   Italia,  178
 (_|_) |)|(()_)()| |   tel.  +39  050 46380
   |   |               picosoft@picosoft.it
 
- picoODBC - picoSQL ODBC driver
+ Copyright (C) Picosoft s.r.l. 1995-2002
 
- Copyright (C) Picosoft s.r.l. 1994-2002
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 2, or (at your option)
+ any later version.
 
- This library is free software; you can redistribute it and/or
- modify it under the terms of the GNU Lesser General Public
- License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
-
- This library is distributed in the hope that it will be useful,
+ This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- Lesser General Public License for more details.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
- You should have received a copy of the GNU Lesser General Public
- License along with this library; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 /*
  * ATTENZIONE!! i puntatori ora sono passati direttamente: questo
  * funziona solo su macchine che hanno i puntatori di 4 bytes
  */
-
+# ifdef WIN32
+# include <windows.h>
+# include <sql.h>
+# include <sqlext.h>
+# else
 # include "sqltype.h"
+# endif
 #ifdef __cplusplus
 extern "C" {
 # include <stdlib.h>
@@ -75,6 +79,7 @@ typedef struct tCommAttr {
 extern CORBA_TCKindDesc BindColOut_seq;
 extern CORBA_TCKindDesc TCD_any_seq;
 extern CORBA_TCKindDesc CommAttr_seq;
+extern CORBA_TCKindDesc UShortSeq;
 
 extern CORBA_TCKindDesc * anyType (HSTMT , unsigned short , short *);
 
@@ -99,11 +104,11 @@ i_SQLError (void **argv)
 void
 i_SQLTables (void **argv)
 {
-   argv[0] = malloc(sizeof (CORBA_Long));
    unsigned char *szTableQualifier = (unsigned char *) argv[2];
    unsigned char *szTableOwner = (unsigned char *) argv[4];
    unsigned char *szTableName = (unsigned char *) argv[6];
    unsigned char *szTableType = (unsigned char *) argv[8];
+   argv[0] = malloc(sizeof (CORBA_Long));
    if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
        szTableQualifier = (unsigned char *) 0;
    if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
@@ -288,15 +293,24 @@ i_SQLFetch (void **argv)
 void
 i_SQLExtendedFetch (void **argv)
 {
+   CORBA_Sequence *seqOut;
+   unsigned long rowSetSize;
    argv[0] = malloc(sizeof (CORBA_Long));
    argv[4] = malloc(sizeof (CORBA_ULong));
-   argv[5] = malloc(sizeof (CORBA_UShort));
+   // argv[5] = malloc(sizeof (CORBA_UShort));
    argv[6] = malloc(sizeof (CORBA_Long));
+
+   
+   if (*((CORBA_Long *) argv[0]) = SQLGetStmtOption (*(HSTMT *) argv[1],
+                                 SQL_ROWSET_SIZE, &rowSetSize) == SQL_ERROR)
+      return;
+   seqOut=(CORBA_Sequence*)(argv[5]=CORBA_Object_new (&UShortSeq, rowSetSize));
+
    *((CORBA_Long *) argv[0]) = SQLExtendedFetch (*(HSTMT *) argv[1],
                                                  *(CORBA_UShort *) argv[2],
                                                  *(CORBA_Long *) argv[3],
                                                   (CORBA_ULong *) argv[4],
-                                                  (CORBA_UShort *) argv[5]);
+                                               (CORBA_UShort *) seqOut->data);
    if (*((CORBA_Long *) argv[0]) == SQL_SUCCESS)
       *((CORBA_Long *) argv[0]) = SQLColAttributes(*(HSTMT *) argv[1],
                                   0, 0, 0, 0, 0, (CORBA_Long *) argv[6]);
@@ -417,10 +431,14 @@ i_SQLGetAllData (void **argv)
       attr->type = fCType;
       Return = SQLGetData (hstmt, i + 1, fCType,buffer,MAXDATALEN,&pcbValue);
       attr->length = pcbValue;
-      if (any->tcIn->kind == tk_Principal) {
+
+      if (pcbValue < 0 || Return == SQL_ERROR) {
+         any->tcIn = &TCD_null;
+         any->data = 0;
+      } else if (any->tcIn->kind == tk_Principal) {
          CORBA_Principal *intObject=
                 (CORBA_Principal *) CORBA_Object_new(&TCD_Principal, pcbValue);
-	 long dummy;
+         long dummy;
 
          if (Return == SQL_SUCCESS_WITH_INFO) {
             memcpy (intObject->data, buffer, MAXDATALEN);
@@ -434,7 +452,7 @@ i_SQLGetAllData (void **argv)
       } else if (any->tcIn->kind == tk_sequence) {
          CORBA_Sequence *intObject=
                 (CORBA_Sequence *) CORBA_Object_new(&TCD_sequence, pcbValue);
-	 long dummy;
+         long dummy;
 
          if (Return == SQL_SUCCESS_WITH_INFO) {
             memcpy (intObject->data, buffer, MAXDATALEN);
@@ -446,20 +464,15 @@ i_SQLGetAllData (void **argv)
          }
          any->data = intObject;
       } else {
-         if (pcbValue >= 0 && Return != SQL_ERROR) {
-            any->data = malloc (pcbValue + 1);
-            memcpy (any->data, buffer, pcbValue);
-            ((char *) any->data)[pcbValue] = 0;
-         } else {
-            any->tcIn = &TCD_null;
-            any->data = 0;
-         }
+         any->data = malloc (pcbValue + 1);
+         memcpy (any->data, buffer, pcbValue);
+         ((char *) any->data)[pcbValue] = 0;
       }
       if (Return != SQL_ERROR) {
-/*	      
+/*      
          SQLColAttributes(hstmt, i + 1, SQL_COLUMN_LENGTH,
                                         0, 0, 0, &(attr->length));
-*/	      
+*/      
          SQLColAttributes(hstmt, i + 1, SQL_COLUMN_PRECISION,
                                         0, 0, 0, &(attr->precision));
          SQLColAttributes(hstmt, i + 1, SQL_COLUMN_SCALE,
@@ -577,11 +590,11 @@ i_SQLColAttributes (void **argv)
 void
 i_SQLColumns (void **argv)
 {
-   argv[0] = malloc(sizeof (CORBA_Long));
    unsigned char *szTableQualifier = (unsigned char *) argv[2];
    unsigned char *szTableOwner = (unsigned char *) argv[4];
    unsigned char *szTableName = (unsigned char *) argv[6];
-   unsigned char *szTableColumnName = (unsigned char *) argv[8];
+   unsigned char *szColumnName = (unsigned char *) argv[8];
+   argv[0] = malloc(sizeof (CORBA_Long));
    if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
        szTableQualifier = (unsigned char *) 0;
    if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
@@ -589,7 +602,7 @@ i_SQLColumns (void **argv)
    if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
        szTableName = (unsigned char *) 0;
    if (*(CORBA_Short *) argv[9] == SQL_NULL_DATA)
-       szTableColumnName = (unsigned char *) 0;
+       szColumnName = (unsigned char *) 0;
    *((CORBA_Long *) argv[0]) = SQLColumns (*(HSTMT *) argv[1],
                                             szTableQualifier,
                                            *(CORBA_Short *) argv[3],
@@ -597,7 +610,7 @@ i_SQLColumns (void **argv)
                                            *(CORBA_Short *) argv[5],
                                             szTableName,
                                            *(CORBA_Short *) argv[7],
-                                            szTableColumnName,
+                                            szColumnName,
                                            *(CORBA_Short *) argv[9]);
 }
 
@@ -665,10 +678,10 @@ i_SQLTransact (void **argv)
 void
 i_SQLStatistics (void **argv)
 {
-   argv[0] = malloc(sizeof (CORBA_Long));
    unsigned char *szTableQualifier = (unsigned char *) argv[2];
    unsigned char *szTableOwner = (unsigned char *) argv[4];
    unsigned char *szTableName = (unsigned char *) argv[6];
+   argv[0] = malloc(sizeof (CORBA_Long));
    if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
        szTableQualifier = (unsigned char *) 0;
    if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
@@ -676,11 +689,11 @@ i_SQLStatistics (void **argv)
    if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
        szTableName = (unsigned char *) 0;
    *((CORBA_Long *) argv[0]) = SQLStatistics (*(HSTMT *) argv[1],
-                                           szTableQualifier ,
+                                           szTableQualifier,
                                           *(CORBA_Short *) argv[3],
                                            szTableOwner,
                                           *(CORBA_Short *) argv[5],
-                                           szTableName ,
+                                           szTableName,
                                           *(CORBA_Short *) argv[7],
                                           *(CORBA_UShort *) argv[8],
                                           *(CORBA_UShort *) argv[9]);
@@ -689,10 +702,10 @@ i_SQLStatistics (void **argv)
 void
 i_SQLSpecialColumns (void **argv)
 {
-   argv[0] = malloc(sizeof (CORBA_Long));
    unsigned char *szTableQualifier = (unsigned char *) argv[3];
    unsigned char *szTableOwner = (unsigned char *) argv[5];
    unsigned char *szTableName = (unsigned char *) argv[7];
+   argv[0] = malloc(sizeof (CORBA_Long));
    if (*(CORBA_Short *) argv[4] == SQL_NULL_DATA)
        szTableQualifier = (unsigned char *) 0;
    if (*(CORBA_Short *) argv[6] == SQL_NULL_DATA)
@@ -732,6 +745,16 @@ i_SQLGetCursorName (void **argv)
                                                 (CORBA_Short *) argv[4]);
 }
 
+void
+i_SQLSetCursorName (void **argv)
+{
+   argv[0] = malloc(sizeof (CORBA_Long));
+
+   *((CORBA_Long *) argv[0]) = SQLSetCursorName (*(HSTMT *) argv[1],
+                                                (unsigned char *) argv[2],
+                                               *(CORBA_Short *) argv[3]);
+}
+
 void
 i_SQLPutData (void **argv)
 {
@@ -755,8 +778,8 @@ i_SQLNumParams (void **argv)
 void
 i_SQLBindParameter (void **argv)
 {
-   argv[0] = malloc(sizeof (CORBA_Long));
    CORBA_Any *any = (CORBA_Any *) argv[8];
+   argv[0] = malloc(sizeof (CORBA_Long));
    any->tcIn = any->tcOut;
    Log ("in BindParameter #2 [%d]", *(CORBA_UShort *)  argv[2]);
    Log ("in BindParameter #3 [%d]", *(CORBA_Short *)  argv[3]);
@@ -778,3 +801,185 @@ i_SQLBindParameter (void **argv)
                                            *(CORBA_Long *)  argv[9],
                                             (CORBA_Long *)  argv[10]);
 }
+
+void
+i_SQLTablePrivileges (void **argv)
+{
+   unsigned char *szTableQualifier = (unsigned char *) argv[2];
+   unsigned char *szTableOwner = (unsigned char *) argv[4];
+   unsigned char *szTableName = (unsigned char *) argv[6];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szTableQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szTableOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szTableName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLTablePrivileges (*(HSTMT *) argv[1],
+                                           szTableQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szTableOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szTableName,
+                                          *(CORBA_Short *) argv[7]);
+}
+
+void
+i_SQLColumnPrivileges (void **argv)
+{
+   unsigned char *szTableQualifier = (unsigned char *) argv[2];
+   unsigned char *szTableOwner = (unsigned char *) argv[4];
+   unsigned char *szTableName = (unsigned char *) argv[6];
+   unsigned char *szColumnName = (unsigned char *) argv[8];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szTableQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szTableOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szTableName = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[9] == SQL_NULL_DATA)
+       szColumnName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLColumnPrivileges (*(HSTMT *) argv[1],
+                                           szTableQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szTableOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szTableName,
+                                          *(CORBA_Short *) argv[7],
+                                           szColumnName,
+                                          *(CORBA_Short *) argv[9]);
+}
+
+void
+i_SQLPrimaryKeys (void **argv)
+{
+   unsigned char *szTableQualifier = (unsigned char *) argv[2];
+   unsigned char *szTableOwner = (unsigned char *) argv[4];
+   unsigned char *szTableName = (unsigned char *) argv[6];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szTableQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szTableOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szTableName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLPrimaryKeys (*(HSTMT *) argv[1],
+                                           szTableQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szTableOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szTableName,
+                                          *(CORBA_Short *) argv[7]);
+}
+
+void
+i_SQLForeignKeys (void **argv)
+{
+   unsigned char *szPkTableQualifier = (unsigned char *) argv[2];
+   unsigned char *szPkTableOwner = (unsigned char *) argv[4];
+   unsigned char *szPkTableName = (unsigned char *) argv[6];
+   unsigned char *szFkTableQualifier = (unsigned char *) argv[8];
+   unsigned char *szFkTableOwner = (unsigned char *) argv[10];
+   unsigned char *szFkTableName = (unsigned char *) argv[12];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szPkTableQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szPkTableOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szPkTableName = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[9] == SQL_NULL_DATA)
+       szFkTableQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[11] == SQL_NULL_DATA)
+       szFkTableOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[13] == SQL_NULL_DATA)
+       szFkTableName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLForeignKeys (*(HSTMT *) argv[1],
+                                           szPkTableQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szPkTableOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szPkTableName,
+                                          *(CORBA_Short *) argv[7],
+                                           szFkTableQualifier,
+                                          *(CORBA_Short *) argv[9],
+                                           szFkTableOwner,
+                                          *(CORBA_Short *) argv[11],
+                                           szFkTableName,
+                                          *(CORBA_Short *) argv[13]);
+}
+
+void
+i_SQLProcedures (void **argv)
+{
+   unsigned char *szProcQualifier = (unsigned char *) argv[2];
+   unsigned char *szProcOwner = (unsigned char *) argv[4];
+   unsigned char *szProcName = (unsigned char *) argv[6];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szProcQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szProcOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szProcName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLProcedures (*(HSTMT *) argv[1],
+                                           szProcQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szProcOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szProcName,
+                                          *(CORBA_Short *) argv[7]);
+}
+
+void
+i_SQLProcedureColumns (void **argv)
+{
+   unsigned char *szProcQualifier = (unsigned char *) argv[2];
+   unsigned char *szProcOwner = (unsigned char *) argv[4];
+   unsigned char *szProcName = (unsigned char *) argv[6];
+   unsigned char *szColumnName = (unsigned char *) argv[8];
+   argv[0] = malloc(sizeof (CORBA_Long));
+   if (*(CORBA_Short *) argv[3] == SQL_NULL_DATA)
+       szProcQualifier = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[5] == SQL_NULL_DATA)
+       szProcOwner = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[7] == SQL_NULL_DATA)
+       szProcName = (unsigned char *) 0;
+   if (*(CORBA_Short *) argv[9] == SQL_NULL_DATA)
+       szColumnName = (unsigned char *) 0;
+   *((CORBA_Long *) argv[0]) = SQLProcedureColumns (*(HSTMT *) argv[1],
+                                           szProcQualifier,
+                                          *(CORBA_Short *) argv[3],
+                                           szProcOwner,
+                                          *(CORBA_Short *) argv[5],
+                                           szProcName,
+                                          *(CORBA_Short *) argv[7],
+                                           szColumnName,
+                                          *(CORBA_Short *) argv[9]);
+}
+
+void
+i_SQLDescribeParam (void **argv)
+{
+   argv[0] = malloc(sizeof (CORBA_Long));
+   argv[3] = malloc(sizeof(CORBA_Short));
+   argv[4] = malloc(sizeof(CORBA_ULong));
+   argv[5] = malloc(sizeof(CORBA_Short));
+   argv[6] = malloc(sizeof(CORBA_Short));
+
+   *((CORBA_Long *) argv[0]) = SQLDescribeParam (*(HSTMT *) argv[1],
+                                           *(CORBA_UShort *)argv[2],
+                                            (CORBA_Short *) argv[3],
+                                            (CORBA_ULong *) argv[4],
+                                            (CORBA_Short *) argv[5],
+                                            (CORBA_Short *) argv[6]);
+}
+
+void
+i_SQLMoreResults (void **argv)
+{
+   argv[0] = malloc(sizeof (CORBA_Long));
+
+   *((CORBA_Long *) argv[0]) = SQLMoreResults (*(HSTMT *) argv[1]);
+}
